#!/usr/local/bin/perl -w
#
# Given a news article or mbox file of news articles on stdin,
# issue a NoCeM notice for those articles
#
# Copyright (c) 1998 by Alan Schwartz
#
use strict 'subs';
use strict 'refs';
use English;
use Mail::Header;
require 'ctime.pl';
require 5;

#######################################################################
# USER CONFIGURATION: VARIABLES YOU MUST SET
#######################################################################

# Your email address
my $issuer = 'myname@myhost';

# Location of the pgp program:

my $pgp = '/usr/local/bin/pgp';

# The id of the PGP key to sign messages with
my $pgpid = 'myname@myhost';

# Put your PGP key pass phrase here if you don't want to be
# prompted for it.
$ENV{'PGPPASS'} = 'pgp pass phrase here';

# The tag to go at the end of the Subject: header:
# Use "@@NCM" if you're hiding spam/spew
# Use "@@<newsgroup name>" if you're hiding off-topic postings
my $subjecttag = '@@NCM';

# The "type" of notice. Example: spam, spew, mmf
my $type = "spam";

# You can put an explanation here of the purpose of these notices. 
# The paragraph must end with the string EOP, on a line by itself.
my $explanation = <<'EOP';
This is an automatically generated NoCeM ("no-see-um") notice.  It is
purely advisory and does not cancel the message listed below. It's a
convenience for Usenet readers who may choose to mark these articles as
"already read".
EOP

#######################################################################
# END OF USER CONFIGURATION
#######################################################################

sub read_mbox_stdin;

my @messages = read_mbox_stdin();
die "No articles to process\en" unless @messages;
my @nocems;
my $newsgroup;
my $id = "$issuer-$$-" . time;
unless ($subjecttag eq '@@NCM') {
  $newsgroup = $subjecttag;
  $newsgroup =~ s/\e@\e@//;
}
my $nltab = "\en\et";
foreach my $msgref (@messages) {
  my $header = new Mail::Header $msgref, MailFrom => 'IGNORE';
  my $messageid = $header->get("Message-ID");
  chomp($messageid);
  my $newsgroups = $header->get("Newsgroups");
  chomp($newsgroups);
  $newsgroups =~ s/,/$nltab/og;
  push(@nocems,"$messageid\et$newsgroups\en");
}

my $headers = <<EOP;
Newsgroups: news.lists.filters
From: $issuer
Subject: NoCeM notice $id $subjecttag

EOP

my $tempfile = "/tmp/mknocem.$$";
open(OUT,">$tempfile") or 
  die "Unable to open temporary file $tempfile: $!\en";
print OUT <<EOP;
$explanation

\e@\e@BEGIN NCM HEADERS
Version: 0.93
Issuer: $issuer
Type: $type
Action: hide
EOP

print OUT "Newsgroup: $newsgroup\en" if $newsgroup;
print OUT "Count: ", scalar(@nocems), "\en";
print OUT '@@BEGIN NCM BODY', "\en";
print OUT join("",@nocems);
print OUT '@@END NCM BODY', "\en";
close(OUT);

system("$pgp -s $tempfile -u $pgpid");
open(OUT,"$tempfile.asc") or 
  die "Unable to open pgp-signed temporary file $tempfile.asc: $!\en";
print $headers;
print <OUT>;
close(OUT);
unlink("$tempfile");
unlink("$tempfile.asc");
exit 0;


# Based on read_mbox in Graham Barr's MailTools module
# Returns an array of references to messages, where each
# message is an array of lines.
sub read_mbox_stdin {
 my @mail  = ();
 my $mail  = [];
 my $blank = 1;

 while(<STDIN>) {
  if($blank && /\eAFrom .*\ed{4}/) {
   push(@mail, $mail) if scalar(@{$mail});
   $mail = [ $_ ];
   $blank = 0;
  } else {
   $blank = m#\eA\eZ#o ? 1 : 0;
   push(@{$mail}, $_);
  }
 }
 push(@mail, $mail) if scalar(@{$mail});
 return wantarray ? @mail : \e@mail;
}


